/* Práce SOČ Jan Šlehofer E4.B
 *  
 * Ampalyzer v1.0
 * Program poloautomatickeho pristroje pro mereni na zvukovych zesilovacich.
 * Urceno pro procesor STM32F103CB
 */
 
//knihovny
#include <Adafruit_ILI9341_STM.h>
#include <Adafruit_GFX_AS.h>
#include <X9C.h>
#include <SPI.h>
#include "SdFat.h"
#include "XPT2046_touch.h"
#include <MD_AD9833.h>

//X9C
//uroven signalniho generatoru
X9C gLv;
#define gLvINC PB4
#define gLvUD PB3
#define gLvCS PB6
byte genLevel = 0;
byte curgenLevel = 255;
//vstupni automaticke prepinani rozsahu
X9C aLv;
#define aLvINC PB3
#define aLvUD PB4
#define aLvCS PB8
byte aLevel = 0;
//pocet kroku vstupniho zesileni
#define aSteps 30

//mereni proudu
float ampDiv[2] = {0.4129, 7.3416};
bool ampRange = 0;
bool curampRange = 1;
//mereni napeti
//                       0  1  2   3   4   5   6   7   8   9   10  11  12  13  14  15   16   17   18   19   20   21   22   23   24   25   26   27   28   29
byte aLvTable[aSteps] = {0, 4, 10, 16, 24, 32, 38, 46, 54, 62, 70, 78, 84, 90, 96, 102, 108, 114, 120, 126, 134, 142, 150, 158, 166, 174, 180, 186, 192, 199};
//                             0            1            2            3            4            5            6            7           8           9            10           11           12           13           14           15           16           17           18          19           20           21           22           23          24           25           26           27           28           29
float aVoltDivTable[aSteps] = {3.594204532, 3.154053511, 2.677330157, 2.309469467, 1.923975777, 1.629514448, 1.445851177, 1.24449592, 1.06071042, 0.904531972, 0.773383434, 0.648492391, 0.559604524, 0.554493861, 0.567577568, 0.509044233, 0.426832066, 0.373107059, 0.32523439, 0.285234643, 0.243300399, 0.209255134, 0.179698328, 0.15522624, 0.131322153, 0.112507159, 0.097636775, 0.085631925, 0.073759828, 0.058720036};
float calibrate = 1.0;

bool inputRange = 0;
bool curinputRange = 1;
//AD9833 generator
#define gData PA15
#define gClk PA12
#define gFsync PA11
#define gTriggerPin PB12
MD_AD9833 gen(gData, gClk, gFsync);
//korigovana hodnota pro nastaveni referencni frekvence 1kHz
#define k_ref 1000.1
bool genRange = 0;
byte genMode = 0;
byte curgenMode = 255;
float genFreq = 0;
float curgenFreq = 99999.0;

//rychle ADC
#define BKP_REG_BASE   (uint32_t *)(0x40006C00 +0x04)
#define analogInPin PA3
#define analogInPinI PA2
#define vTriggerPin PA4
#define aTriggerPin PA1
//pocet vzorku
#define maxSamples 1024*5
uint16_t nullpoint = 2048;
uint16_t nullAmp = 2048;
//pole pro data z ADC
int32_t dataPoints32[maxSamples / 2];
int16_t *dataPoints = (int16_t *)&dataPoints32;
//konec DMA indikace
volatile static bool dma1_ch1_Active;
#define ADC_CR1_FASTINT 0x70000 // Fast interleave rezim

//SD
SdFat SD(2);
SdFile file;

#define FILE_BASE_NAME "Analyz"
#define SD_CS PA8
bool SDon = 0;

//dotyk
#define Touch_CS PB10
SPIClass SPI_2(2);
XPT2046_touch ts(Touch_CS, SPI_2);
uint16_t xy[2];
unsigned int TTLimit = 0;
bool touchPressed = 0;
//kalibrace dotyku
#define TS_MINX 270
#define TS_MINY 270
#define TS_MAXX 3800
#define TS_MAXY 3800

//TFT display
#define TFT_CS PC13
#define TFT_DC PB0
#define TFT_RST PB1
Adafruit_ILI9341_STM tft = Adafruit_ILI9341_STM(TFT_CS, TFT_DC, TFT_RST);
byte page = 0;
byte TFTpage = 255;
unsigned int TFTLimit = 0;
unsigned long UPScalc = 0;

//registr rozsireni vystupu
#define regData PB4
#define regClk PB3
#define regLoad PB5
byte regOut = 0;
//vystupy
#define gen_en 0 //generator enable
#define gen_rg 1  //generator range (1-10 / 0,01 - 1)
#define gen_md 2  //generator mode
#define in_md 3   //input mode
#define in_aR 4   //input current range
#define in_vR 5   //input voltage range
#define in_en 6  //input enable voltage

//multimetr
uint32_t multiLimit = 0;
bool multiTime = 0;

double DCvolts = 0;
double ACrmsvolts = 0;

double DCamps = 0;
double ACrmsamps = 0;

double Watt = 0;

//osciloskop
//parametry stinitka
#define RetColor ILI9341_WHITE
#define beamColor ILI9341_YELLOW
#define RetWidth 240
#define RetHeight 180
#define RetStartY 45
bool osciSource = 0;
bool curosciSource = 1;

//              casova zakladna 0                   , 1
byte osciDisplay[119];      //  0   1   2   3    4    5    6     7     8
uint16_t osciTimeDivTable[9] = {10, 20, 50, 100, 200, 500, 1000, 2000, 5000}; //10us - 5000us(5ms), 2 5 10 krok, 9 kroku
byte osciTimeDiv = 0;
byte curosciTimeDiv = 255;
//             vertikalni kanal 0   1    2    3    4     5     6
uint16_t osciVoltDivTable[7] = {50, 100, 200, 500, 1000, 2000, 5000}; //50mV - 5000mV(5V), 2 5 10 krok, 7 kroku
byte osciVoltLevelTable[7] = {  0,  4,   9,   15,  19,   24,   28};
byte osciVoltDiv = 3;
byte curosciVoltDiv = 255;

byte tB = 0;
double sRateTable[2] = {6.94, 124.8786};

//analyza harmonickeho zkresleni
bool thdTime = 0;

float distort = 0;
float distortN = 0;

float freq = 0;
float duty = 0;
float period = 0;

//poloautomaticke mereni
#define separ ";"
bool testRunA = 0;
bool testRunB = 0;

float gain = 0;
float logScale[10] = {1, 1.3, 1.7, 2.2, 2.8, 3.6, 4.6, 6, 7.7, 10};

//procesy pri spusteni
void setup() {
  Serial.begin(115200);
  delay(100);
  disableDebugPorts();
  delay(100);
  IOinit();
  touchTFTInit();
  SDInit();
  genInit();
  inputInit();
  delay(100);
  selfCalibrate();
  tft.fillScreen(ILI9341_BLACK);
}

//hlavni cyklus
void loop() {
  TFTLoop();
  touchLoop();
  genLoop();
}

//inicializace registru vystupniho rozsireni
void IOinit() {
  pinMode(regData, OUTPUT);
  pinMode(regClk, OUTPUT);
  pinMode(regLoad, OUTPUT);
  digitalWrite(regLoad, HIGH);
}

//inicializace dotykového displaye
void touchTFTInit() {
  pinMode(Touch_CS, OUTPUT);
  digitalWrite(Touch_CS, HIGH);
  tft.begin();
  Serial.println("TFT uspesne spusten");
  Serial.println("Dotykova obrazovka spustena");
  tft.setRotation(4);
  tft.fillScreen(ILI9341_BLACK);
  //logo
  tft.setCursor(15, 120);
  tft.setTextColor(ILI9341_YELLOW);
  tft.setTextSize(4);
  tft.print("Ampalyzer");
  tft.setTextSize(3);
  tft.setCursor(15, 190);
  tft.print("DMP Slehofer");
  tft.setCursor(80, 220);
  tft.print("E4.B");
  tft.setCursor(80, 250);
  tft.print("SPSE");
  tft.setCursor(40, 280);
  tft.print("Uzlabina");
}

//inicializace pripojeni karty SD
void SDInit() {
  Serial.print("Zavadim SD kartu...");
  if (!SD.begin(SD_CS)) {
    Serial.println("zavadeni selhalo!");
    SDon = 0;
    return;
  }
  Serial.println("zavadeni uspesne");
  SDon = 1;
  const uint8_t BASE_NAME_SIZE = sizeof(FILE_BASE_NAME) - 1;
  char fileName[13] = FILE_BASE_NAME "00.csv";
  
  //nalezeni nepouzivaneho nazvu souboru
  while (SD.exists(fileName)) {
    if (fileName[BASE_NAME_SIZE + 1] != '9') {
      fileName[BASE_NAME_SIZE + 1]++;
    } 
    else if (fileName[BASE_NAME_SIZE] != '9') {
      fileName[BASE_NAME_SIZE + 1] = '0';
      fileName[BASE_NAME_SIZE]++;
    } 
    else {
      Serial.println("nelze vytvorit soubor");
    }
  }
  if (!file.open(fileName, O_CREAT | O_WRITE | O_EXCL)) {
    Serial.println("chyba otevreni souboru");
  }
  Serial.print(fileName);
  Serial.println("soubor vytvoren uspesne");
}

//inicializace generatoru
void genInit() {
  gen.begin();
  pinMode(gTriggerPin, INPUT);
  regWrite(gen_en, 1);
  gen.setMode(MD_AD9833::MODE_OFF);
  gen.setPhase(MD_AD9833::CHAN_0, 0);
  gen.setFrequency(MD_AD9833::CHAN_0, 0);
  gLv.begin(gLvCS, gLvINC, gLvUD);
  gLv.setPot(0, 0);
}

//inicializace vstupního modulu
void inputInit() {
  pinMode(analogInPin, INPUT_ANALOG);
  pinMode(analogInPinI, INPUT_ANALOG);
  pinMode(vTriggerPin, INPUT);
  pinMode(aTriggerPin, INPUT);
  regWrite(in_en, 0);
  aLv.begin(aLvCS, aLvINC, aLvUD);
  aLv.setPot(0, 0);
  adc_calibrate(ADC1);
  adc_calibrate(ADC2);
}

//cyklus generatoru
void genLoop() {
  if (genMode != curgenMode) {
    switch (genMode) {
      case (0):
        gen.setMode(MD_AD9833::MODE_OFF);
        regWrite(gen_en, 0);
        regWrite(gen_md, 0);
        gen.setFrequency(MD_AD9833::CHAN_0, genFreq);
        break;
      case (1):
        gen.setMode(MD_AD9833::MODE_SINE);
        regWrite(gen_en, 1);
        regWrite(gen_md, 0);
        gen.setFrequency(MD_AD9833::CHAN_0, genFreq);
        break;
      case (2):
        gen.setMode(MD_AD9833::MODE_TRIANGLE);
        regWrite(gen_en, 1);
        regWrite(gen_md, 0);
        gen.setFrequency(MD_AD9833::CHAN_0, genFreq);
        break;
      case (3):
        gen.setMode(MD_AD9833::MODE_SINE);
        regWrite(gen_en, 1);
        regWrite(gen_md, 1);
        gen.setFrequency(MD_AD9833::CHAN_0, k_ref);
        curgenFreq = k_ref;
        break;
    }
    curgenMode = genMode;
  }
  if (genFreq != curgenFreq && genMode != 3) {
    gen.setFrequency(MD_AD9833::CHAN_0, genFreq);
    curgenFreq = genFreq;
  }
  if (genLevel != curgenLevel) {
    curgenLevel = genLevel;
    if (genLevel == 0) {
      gLv.setPot(0, 0);
      regWrite(gen_en, 0);
    }
    else {
      if (genMode != 0) regWrite(gen_en, 1);
      if (genLevel > 100){
        regWrite(gen_rg, 1);
        genRange = 1;
      }
      else {
        regWrite(gen_rg, 0);
        genRange = 0;
      }
      gLv.setPot(genLevel - 1 - (genRange * 100), 0);
    }
  }
}

//cyklus displaye
void TFTLoop() {
  if (millis() < (TFTLimit + 100)) return;
  TFTLimit = millis();

  //dynamicke prvky
  uint32_t divi = 0;
  
  switch (page) {
    //hlavni stranka
    case (0):
      //tlacitko generatoru
      tft.fillRect(10, 40, 100, 20, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(2);
      tft.setCursor(10, 40);
      switch (genMode) {
        case (0):
          tft.print("OFF");
          break;
        case (1):
          tft.print("SINUS");
          break;
        case (2):
          tft.print("TROJU");
          break;
        case (3):
          tft.print("1k REF");
          break;
      }
      break;
    //stranka generatoru
    case (1):
      //nastaveni rezimu
      tft.setCursor(119, 65);
      tft.fillRect(119, 65, 120, 25, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      switch (genMode) {
        case (0):
          tft.print("OFF");
          break;
        case (1):
          tft.print("SINUS");
          break;
        case (2):
          tft.print("TROJU");
          break;
        case (3):
          tft.print("1k REF");
          break;
      }

      //nastaveni frekvence
      divi = 1000000;
      tft.setTextColor(ILI9341_WHITE);
      tft.fillRect(10, 160, 230, 25, ILI9341_BLACK);
      tft.setCursor(187, 160);
      tft.print('.');
      for (int i = 0; i < 6; i++) {
        tft.setCursor(10 + i * 40, 160);
        uint32_t v = (genFreq * 10.00) / divi;
        v *= divi;
        v = (genFreq * 10.00) - v;
        divi /= 10;
        v /= divi;
        tft.print(v);
      }
      //nastaveni vystupni urovne
      tft.setTextColor(ILI9341_WHITE);
      tft.fillRect(110, 253, 100, 25, ILI9341_BLACK);
      if (genRange == 1) tft.setCursor(170, 253);
      else tft.setCursor(130, 253);
      tft.print('.');
      divi = 1000;
      for (int i = 0; i < 3; i++) {
        tft.setCursor(110 + i * 40, 253);
        uint32_t v = (genLevel - 100 * genRange) / divi;
        v *= divi;
        v = (genLevel - 100 * genRange) - v;
        divi /= 10;
        v /= divi;
        tft.print(v);
      }
      break;
    //multimetr
    case (2):
      //nacteni hodnot
      if (millis() > (multiLimit + 1000)) {
        multiLimit = millis();
        multiSampleVal(multiTime);
        //voltage
        tft.fillRect(60, 90, 180, 70, ILI9341_BLACK);
        //current
        tft.fillRect(60, 210, 180, 80, ILI9341_BLACK);
        //wattage
        tft.fillRect(85, 290, 155, 30, ILI9341_BLACK);
      }
      
      tft.setTextColor(ILI9341_YELLOW);
      tft.setTextSize(3);

      //casova zakladna
      tft.fillRect(145, 50, 22, 22, ILI9341_BLUE);
      tft.setTextSize(3);
      tft.setCursor(145, 50);
      if (multiTime == 1) tft.print('S');
      else tft.print('F');
      
      //uroven
      tft.fillRect(203, 50, 37, 30, ILI9341_BLACK);
      tft.setCursor(203, 50);
      tft.print(aLevel);
      
      //stejnosmerna stredni hodnota
      tft.setCursor(60, 90);
      if (DCvolts > 999.00 || DCvolts < -999.00) {
        tft.print(DCvolts / 1000.0, 2);
        tft.print("V");
      }
      else {
        tft.print(DCvolts, 1);
        tft.print("mV");
      }
      
      //stridava hodnota rms
      tft.setCursor(60, 130);
      if (ACrmsvolts > 999.00 || ACrmsvolts < -999.00) {
        tft.print(ACrmsvolts / 1000.0, 2);
        tft.print("V");
      }
      else {
        tft.print(ACrmsvolts, 1);
        tft.print("mV");
      }
      
      //stejnosmerna stredni hodnota
      tft.setCursor(60, 210);
      if (DCamps > 999.00 || DCamps < -999.00) {
        tft.print(DCamps / 1000.0, 2);
        tft.print("A");
      }
      else {
        tft.print(DCamps, 1);
        tft.print("mA");
      }
      
      //stridava hodnota rms
      tft.setCursor(60, 250);
      if (ACrmsamps > 999.00 || ACrmsamps < -999.00) {
        tft.print(ACrmsamps / 1000.0, 2);
        tft.print("A");
      }
      else {
        tft.print(ACrmsamps, 1);
        tft.print("mA");
      }
      //vykon do zateze
      tft.setCursor(85, 290);
      if (Watt > 999.00 || Watt < -999.00) {
        tft.print(Watt / 1000.0, 2);
        tft.print("W");
      }
      else {
        tft.print(Watt, 1);
        tft.print("mW");
      }
      
      //rozsah ampermetru
      tft.fillRect(210, 170, 22, 22, ILI9341_BLUE);
      tft.setTextSize(3);
      tft.setCursor(210, 170);
      if (ampRange == 1) tft.print('L');
      else tft.print('H');
    break;
    //osciloskop
    case (3):
      //casova zakladna
      tft.setTextColor(ILI9341_WHITE);
      tft.setTextSize(2);
      if(osciTimeDiv != curosciTimeDiv){
        curosciTimeDiv = osciTimeDiv;
        
        tft.fillRect(30, 230, 57, 30, ILI9341_BLACK);
        tft.setCursor(32, 238);
        if(osciTimeDiv > 2){
          tft.print((float)osciTimeDivTable[osciTimeDiv] / 1000.0, 1);
          tft.print("ms");
        }
        else {
          tft.print(osciTimeDivTable[osciTimeDiv]);
          tft.print("us");
        }
      }
      
      //vertikalni kanal
      if(curosciVoltDiv != osciVoltDiv){
        tft.fillRect(153, 230, 57, 30, ILI9341_BLACK);
        tft.setCursor(158, 238);
        if(osciVoltDiv > 0){
          tft.print((float)osciVoltDivTable[osciVoltDiv] / 1000.0, 1);
          tft.print("V");
        }
        else {
          tft.print(osciVoltDivTable[osciVoltDiv]);
          tft.print("mV");
        }
      }
      tft.setTextSize(3);
      
      osciSampleVal();
      drawGraticule();
      drawOsci();
      
      //volba zdroje    
      if (osciSource != curosciSource) {
        regWrite(in_md, osciSource);
        curosciSource = osciSource;
        tft.setCursor(119, 285);
        tft.fillRect(119, 285, 120, 25, ILI9341_YELLOW);
        tft.setTextColor(ILI9341_BLACK);
        if (osciSource == 1) tft.print("EXT");
        else tft.print("GEN");
      }
      break;
    //analyzator zkresleni
    case (4):
      regWrite(in_en, 1);
      delayMicroseconds(500);
      autoRangevolt(thdTime);
      takeSamples(0, thdTime, 2);
      getFreq();
      
      regWrite(in_en, 0);
      tft.setTextColor(ILI9341_YELLOW);
      tft.setCursor(0, 230);
      
      distort = THDanalyze(20, 0);
      distortN = THDanalyze(20, 1);
      
      //zkresleni
      tft.fillRect(60, 90, 180, 70, ILI9341_BLACK);
      //frekvence
      tft.fillRect(60, 210, 180, 110, ILI9341_BLACK); 
      
      tft.setTextColor(ILI9341_YELLOW);
      tft.setTextSize(3);

      //casova zakladna
      tft.fillRect(205, 50, 22, 22, ILI9341_BLUE);
      tft.setCursor(205, 50);
      if (thdTime == 1) tft.print('S');
      else tft.print('F');
      
      //zkresleni
      tft.setCursor(65, 90);
      if(distort > 0.099) tft.print(distort * 100.0, 1);
      else tft.print(distort * 100.0, 2);
      tft.print("%");
      /*
      tft.print(20 * log(distort), 0);
      tft.print("dB");
      */
      //zkresleni a sum
      tft.setCursor(65, 130);
      if(distortN > 0.099) tft.print(distortN * 100.0, 1);
      else tft.print(distortN * 100.0, 2);
      tft.print("%");
      /*
      tft.print(20 * log(distortN), 0);
      tft.print("dB");
      */
      //frekvence
      tft.setCursor(60, 210);
      if (freq > 999.00) {
        tft.print(freq / 1000.0, 2);
        tft.print("kHz");
      }
      else {
        tft.print(freq, 1);
        tft.print("Hz");
      }
      
      //strida
      tft.setCursor(60, 250);
      tft.print(duty * 100.0, 2);
      tft.print("%");
      
      //perioda
      tft.setCursor(60, 290);
      if (period > 999.00) {
        tft.print(period / 1000.0, 2);
        tft.print("ms");
      }
      else {
        tft.print(period, 1);
        tft.print("us");
      }
      
      break;
    //poloautomaticke mereni
    case (5):
      if(testRunA == 1 && TFTpage == page && SDon == 1){        
        Ampalyze();
        testRunA = 0;
      }
      else if(TFTpage == page) testRunA = 0;

      if(testRunB == 1 && TFTpage == page && SDon == 1){        
        FreqResp();
        testRunB = 0;
      }
      else if(TFTpage == page) testRunB = 0;
      
    break;
    //nastaveni
    case (6):
      
      break;
  }

  if (page == TFTpage) return;

  //staticke prvky
  tft.fillScreen(ILI9341_BLACK);
  switch (page) {
    //hlavni strana
    case (0):
      //tlacitko generatoru
      tft.fillRoundRect(5, 5, 110, 96, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(3);
      tft.setCursor(10, 10);
      tft.print("GEN");
      //tlacitko multimetru
      tft.fillRoundRect(125, 5, 110, 96, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(3);
      tft.setCursor(130, 10);
      tft.print("MULTI");
      //tlacitko osciloskopu
      tft.fillRoundRect(5, 109, 110, 96, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(3);
      tft.setCursor(10, 114);
      tft.print("OSCI");
      //tlacitko mereni zkresleni
      tft.fillRoundRect(125, 109, 110, 96, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(3);
      tft.setCursor(130, 114);
      tft.print("ZKRES");
      tft.setCursor(130, 145);
      tft.print("FREK");
      //tlacitko poloautomatickeho mereni
      tft.fillRoundRect(5, 213, 110, 97, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(3);
      tft.setCursor(10, 218);
      tft.print("AUTO");
      //tlacitko nastaveni
      tft.fillRoundRect(125, 213, 110, 97, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(3);
      tft.setCursor(130, 218);
      tft.print("NAST.");
      break;
    //generator
    case (1):
      //tlacitko navratu na hlavni stranu
      tft.fillRoundRect(5, 5, 235, 35, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(3);
      tft.setCursor(10, 10);
      tft.print("NAVRAT");
      //tlacitko volby rezimu
      tft.fillRoundRect(5, 55, 235, 35, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(3);
      tft.setCursor(15, 65);
      tft.print("REZIM: ");
      //volba frekvence
      tft.setTextColor(ILI9341_WHITE);
      tft.setTextSize(3);
      tft.setCursor(5, 100);
      tft.print("Frekvence Hz");
      for (int d = 0; d < 2; d++) {
        for (int i = 0; i < 6; i++) {
          tft.fillRoundRect(5 + i * 40, 125 + d * 60, 30, 30, 5, ILI9341_BLUE);
          tft.setCursor(8 + i * 40 + 5, 130 + d * 60);
          if (d == 1) tft.print('-');
          else tft.print('+');
        }
      }
      //volba urovne
      tft.setTextColor(ILI9341_WHITE);
      tft.setTextSize(3);
      tft.setCursor(10, 240);
      tft.print("Ampli");
      tft.setCursor(10, 265);
      tft.print("  V");
      
      for (int d = 0; d < 2; d++) {
        for (int i = 0; i < 3; i++) {
          tft.fillRoundRect(105 + i * 40, 218 + d * 60, 30, 30, 5, ILI9341_BLUE);
          tft.setCursor(108 + i * 40 + 5, 223 + d * 60);
          if (d == 1) tft.print('-');
          else tft.print('+');
        }
      }
      break;
    //multimetr
    case (2):
      VoltNull();
      AmpNull();
      //tlacitko navratu na hlavni stranu
      tft.fillRoundRect(5, 5, 235, 35, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(3);
      tft.setCursor(10, 10);
      tft.print("NAVRAT");
      //hodnoty
      tft.setTextColor(ILI9341_YELLOW);
      tft.setTextSize(3);
      //napeti
      tft.setCursor(5, 50);
      tft.print("Napeti");
      tft.fillRoundRect(140, 45, 35, 35, 5, ILI9341_BLUE);
      tft.setCursor(5, 90);
      tft.print("DC");
      tft.setCursor(5, 130);
      tft.print("AC");
      //proud
      tft.setCursor(5, 170);
      tft.print("Proud");
      tft.fillRoundRect(200, 165, 35, 35, 5, ILI9341_BLUE);
      tft.setCursor(5, 210);
      tft.print("DC");
      tft.setCursor(5, 250);
      tft.print("AC");
      //vykon
      tft.setCursor(5, 290);
      tft.print("Watt");
      break;
    //osciloskop
    case (3):
      VoltNull();
      curosciTimeDiv = 255;
      curosciSource = !osciSource;
      //tlacitko navratu na hlavni stranu
      tft.fillRoundRect(5, 5, 235, 35, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(3);
      tft.setCursor(10, 10);
      tft.print("NAVRAT");
      //ramecek stinitka
      tft.drawRect(0, RetStartY, RetWidth, RetHeight, RetColor);
      
      tft.setTextColor(ILI9341_WHITE);
      //+ cas
      tft.fillRoundRect(0, 230, 30, 30, 5, ILI9341_BLUE);
      tft.setCursor(6, 235);
      tft.print('+');
      //- cas
      tft.fillRoundRect(87, 230, 30, 30, 5, ILI9341_BLUE);
      tft.setCursor(93, 235);
      tft.print('-');
      //+ vertikal
      tft.fillRoundRect(123, 230, 30, 30, 5, ILI9341_BLUE);
      tft.setCursor(129, 235);
      tft.print('+');
      //- vertikal
      tft.fillRoundRect(210, 230, 30, 30, 5, ILI9341_BLUE);
      tft.setCursor(216, 235);
      tft.print('-');

      //volba zdroje
      tft.fillRoundRect(5, 280, 235, 35, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(3);
      tft.setCursor(15, 285);
      tft.print("ZDROJ: ");
      break;
    //analyzator zkresleni
    case (4):
      //tlacitko navratu na hlavni stranu
      tft.fillRoundRect(5, 5, 235, 35, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(3);
      tft.setCursor(10, 10);
      tft.print("NAVRAT");
      //hodnoty
      tft.setTextColor(ILI9341_YELLOW);
      tft.setTextSize(3);
      //zkresleni
      tft.setCursor(5, 50);
      tft.print("Zkresleni");
      tft.fillRoundRect(200, 45, 35, 35, 5, ILI9341_BLUE);
      tft.setCursor(5, 90);
      tft.print("THD");
      tft.setCursor(5, 130);
      tft.print("T+N");
      //Frekvence
      tft.setCursor(5, 170);
      tft.print("Frekvence");
      tft.setCursor(5, 210);
      tft.print("f");
      tft.setCursor(5, 250);
      tft.print("D");
      tft.setCursor(5, 290);
      tft.print("T");
      break;
    //poloautomaticke mereni
    case (5):
      //tlacitko navratu na hlavni stranu
      tft.fillRoundRect(5, 5, 235, 35, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(3);
      tft.setCursor(10, 10);
      tft.print("NAVRAT");

      //tlacitko spusteni testu A
      tft.fillRoundRect(5, 55, 235, 83, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(4);
      tft.setCursor(10, 70);
      tft.print("Ampalyza");

      //tlacitko spusteni testu B
      tft.fillRoundRect(5, 150, 235, 83, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(4);
      tft.setCursor(10, 165);
      tft.print("Freqchar");
      break;
    //nastaveni
    case (6):
      //tlacitko navratu na hlavni stranu
      tft.fillRoundRect(5, 5, 235, 35, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(3);
      tft.setCursor(10, 10);
      tft.print("NAVRAT");
      
      //tlacitko kalibrace
      tft.fillRoundRect(5, 55, 235, 83, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(4);
      tft.setCursor(10, 60);
      tft.print("KALIBR.");
      
      //tlacitko zavedeni karty SD
      tft.fillRoundRect(5, 143, 235, 83, 5, ILI9341_YELLOW);
      tft.setTextColor(ILI9341_BLACK);
      tft.setTextSize(4);
      tft.setCursor(10, 148);
      tft.print("ZAVED. SD");

      tft.setTextColor(ILI9341_YELLOW);
      tft.setTextSize(3);
      tft.setCursor(20, 230);
        
      //stav SD
      tft.fillRect(0, 230, 240, 30, ILI9341_BLACK);
      if(SDon == 1) tft.print("SD zavedena");
      else tft.print("neni SD");
      
      break;
  }
  TFTpage = page;
}

//cyklus dotykové plochy
void touchLoop() {
  if (!ts.read_XY(xy)) {
    touchPressed = 0;
    return;
  }
  if (touchPressed) return;
  touchPressed = 1;
  xy[1] = 25 + map(TS_MAXX - xy[1], TS_MINX, TS_MAXX, 0, tft.width());
  xy[0] = map(xy[0], TS_MINY, TS_MAXY, 0, tft.height());
  xy[1] = tft.width() - xy[1];
  xy[0] = tft.height() - xy[0];
  int x = xy[1];
  xy[1] = xy[0];
  xy[0] = x;

  switch (page) {
    //hlavni strana
    case (0):
      //prvni sloupec
      if (xy[0] > 5 && xy[0] < 115) {
        //generator
        if (xy[1] > 5 && xy[1] < 101) {
          page = 1;
        }
        //osciloskop
        if (xy[1] > 109 && xy[1] < 205) {
          page = 3;
        }
        //poloautomaticke mereni
        if (xy[1] > 213 && xy[1] < 309) {
          page = 5;
        }
      }
      //druhy sloupec
      if (xy[0] > 125 && xy[0] < 235) {
        //multimetr
        if (xy[1] > 5 && xy[1] < 101) {
          page = 2;
        }
        //analyzator zkresleni
        if (xy[1] > 109 && xy[1] < 205) {
          page = 4;
        }
        //nastaveni
        if (xy[1] > 213 && xy[1] < 309) {
          page = 6;
        }
      }
      break;
    //generator
    case (1):
      //navrat na hlavni stranu
      if (xy[1] < 40) {
        page = 0;
      }
      //volba rezimu
      if (xy[1] > 55 && xy[1] < 90) {
        if (genMode == 3) genMode = 0;
        else genMode++;
      }
      //volba frekvence
      //+
      if (xy[1] > 125 && xy[1] < 155) {
        if (xy[0] > 5 && xy[0] < 35 && genFreq <= 89999.9) {
          genFreq += 10000;
        }
        if (xy[0] > 45 && xy[0] < 75 && genFreq <= 98999.9) {
          genFreq += 1000;
        }
        if (xy[0] > 85 && xy[0] < 115 && genFreq <= 99899.9) {
          genFreq += 100;
        }
        if (xy[0] > 125 && xy[0] < 155 && genFreq <= 99989.9) {
          genFreq += 10;
        }
        if (xy[0] > 165 && xy[0] < 195 && genFreq <= 99998.9) {
          genFreq += 1;
        }
        if (xy[0] > 205 && xy[0] < 235 && genFreq <= 99999.8) {
          genFreq += 0.1;
        }
      }
      //-
      if (xy[1] > 185 && xy[1] < 215) {
        if (xy[0] > 5 && xy[0] < 35 && genFreq >= 10000.00) {
          genFreq -= 10000;
        }
        if (xy[0] > 45 && xy[0] < 75 && genFreq >= 1000.00) {
          genFreq -= 1000;
        }
        if (xy[0] > 85 && xy[0] < 115 && genFreq >= 100.00) {
          genFreq -= 100;
        }
        if (xy[0] > 125 && xy[0] < 155 && genFreq >= 10.0) {
          genFreq -= 10;
        }
        if (xy[0] > 165 && xy[0] < 195 && genFreq >= 1.0) {
          genFreq -= 1;
        }
        if (xy[0] > 205 && xy[0] < 235 && genFreq >= 0.1) {
          genFreq -= 0.1;
        }
      }
      //volba urovne
      //+
      if (xy[1] > 218 && xy[1] < 248) {
        if (xy[0] > 105 && xy[0] < 135) {
          if(genLevel > 0){
            genLevel = (110 + 90 * genRange);
            genRange = 1;
          }
          else genLevel = (100 + 100 * genRange);
        }
        if (xy[0] > 145 && xy[0] < 175) {
          if ((genLevel - 100 * genRange) <= 90) genLevel += 10;
          else if (genRange == 0){
            genRange = 1;
            genLevel = 110;
          }
        }
        if (xy[0] > 185 && xy[0] < 215) {
          if ((genLevel - 100 * genRange) <= 99) genLevel += 1;
          else if (genRange == 0) {
            genRange = 1;
            genLevel += 10;
          }
        }
      }
      //-
      if (xy[1] > 278 && xy[1] < 308) {
        if (xy[0] > 105 && xy[0] < 135) {
          genLevel = 100 * genRange;
        }
        if (xy[0] > 145 && xy[0] < 175) {
          if ((genLevel - 100 * genRange) >= 20) genLevel -= 10;
          else if (genRange == 1) {
            genRange = 0;
            genLevel = 100;
          }
          else if ((genLevel - 100 * genRange) >= 10){
            genLevel -= 10;
          }
          else genLevel = 0;
        }
        if (xy[0] > 185 && xy[0] < 215) {
          if (genRange == 1 && (genLevel - 100 * genRange) >= 11) genLevel -= 1;
          else if (genRange == 1) {
            genRange = 0;
            genLevel -= 10;
          }
          else if ((genLevel - 100 * genRange) >= 1){
            genLevel -= 1;
          }
        }
      }
      break;
    //multimetr
    case (2):
      //navrat na hlavni stranu
      if (xy[1] < 40) {
        page = 0;
      }
      //rozsah ampermetru
      if (xy[0] > 200 && xy[0] < 235 && xy[1] > 165 && xy[1] < 200) {
        ampRange = !ampRange;
      }
      //casova zakladna
      if (xy[0] > 140 && xy[0] < 175 && xy[1] > 45 && xy[1] < 80) {
        multiTime = !multiTime;
      }
      break;
    //osciloskop
    case (3):
      //navrat na hlavni stranu
      if (xy[1] < 40) {
        page = 0;
      }
      //zdroj
      if (xy[1] > 280 && xy[1] < 315) {
        osciSource = !osciSource;
      }
      
      if (xy[1] > 230 && xy[1] < 260) {
        //vertikal
        //+
        if (xy[0] > 123 && xy[0] < 153 && osciVoltDiv < 6) {
          osciVoltDiv += 1;
        }
        //-
        if (xy[0] > 210 && xy[0] < 240 && osciVoltDiv > 0) {
          osciVoltDiv -= 1;
        }
        //casova zakladna
        //+
        if (xy[0] > 0 && xy[0] < 30 && osciTimeDiv < 8) {
          osciTimeDiv += 1;
        }
        //-
        if (xy[0] > 87 && xy[0] < 107 && osciTimeDiv > 0) {
          osciTimeDiv -= 1;
        }
      }  
      break;
    //analyzator zkresleni
    case (4):
      //navrat na hlavni stranu
      if (xy[1] < 40) {
        page = 0;
      }
      //casova zakladna
      if (xy[0] > 200 && xy[0] < 235 && xy[1] > 45 && xy[1] < 80) {
        thdTime = !thdTime;
      }
      break;
    //poloautomaticke mereni
    case (5):
      //navrat na hlavni stranu
      if (xy[1] < 40) {
        page = 0;
      }
      //tlacitko spusteni testu A
      if (xy[1] > 55 && xy[1] < 138) {
        testRunA = 1;
        TFTpage++;
      }
      //tlacitko spusteni testu B
      if (xy[1] > 150 && xy[1] < 233) {
        testRunB = 1;
        TFTpage++;
      }
      break;
    //nastaveni
    case (6):
      //navrat na hlavni stranu
      if (xy[1] < 40) {
        page = 0;
      }
      //tlacitko kalibrace
      tft.fillRoundRect(5, 55, 235, 83, 5, ILI9341_YELLOW);
      if (xy[1] > 55 && xy[1] < 138) {
        selfCalibrate();
        TFTpage++;
      }
      //tlacitko zavedeni karty SD
      tft.fillRoundRect(5, 143, 235, 83, 5, ILI9341_YELLOW);
      if (xy[1] > 143 && xy[1] < 226) {
        SDInit();
        TFTpage++;
      }
      break;
  }
}

//poloautomaticky test - vsechny parametry zesilovace
void Ampalyze(){
  bool imped = 0;
  byte impedance = 1;
  
  tft.setTextColor(ILI9341_YELLOW);
  tft.setTextSize(3);

  //spusteni testu
  tft.fillRect(0, 140, 240, 30, ILI9341_BLACK);
  tft.setCursor(10, 140);
  tft.print("Testuji");

  delay(500);

  //obnov stranku
  tft.fillRect(0, 40, 240, 280, ILI9341_BLACK);

  //volba primeho mereni vykonu ci odvozeni od impedance
  tft.setCursor(10, 50);
  tft.print("Merit vykon");
  tft.setCursor(10, 70);
  tft.print("primo");
  //tlacitko ok - primo
  tft.fillRoundRect(0, 95, 60, 40, 5, ILI9341_BLUE);
  tft.setTextColor(ILI9341_WHITE);
  tft.setCursor(6, 100);
  tft.print("OK");

  tft.setTextColor(ILI9341_YELLOW);
  tft.setCursor(10, 145);
  tft.print("odvodit od");
  tft.setCursor(10, 170);
  tft.print("impedance");
  //tlacitko ok - odvodit
  tft.fillRoundRect(0, 195, 60, 40, 5, ILI9341_BLUE);
  tft.setTextColor(ILI9341_WHITE);
  tft.setCursor(6, 200);
  tft.print("OK");

  //zadani impedance
  tft.setTextColor(ILI9341_WHITE);
  tft.setTextSize(3);
  tft.setCursor(10, 240);
  tft.print("Imped");
  tft.setCursor(10, 265);
  tft.print(" ohm");
  
  for (int d = 0; d < 2; d++) {
    for (int i = 0; i < 3; i++) {
      tft.fillRoundRect(105 + i * 40, 218 + d * 60, 30, 30, 5, ILI9341_BLUE);
      tft.setCursor(108 + i * 40 + 5, 223 + d * 60);
      if (d == 1) tft.print('-');
      else tft.print('+');
    }
  }

  bool mode = 1;
  while(mode){
    //zobrazeni zadani hodnot impedance
    if (millis() > (multiLimit + 50)) {
      multiLimit = millis();
      tft.setTextColor(ILI9341_WHITE);
      tft.fillRect(110, 253, 100, 25, ILI9341_BLACK);
      tft.setCursor(170, 253);
      int divi = 1000;
      for (int i = 0; i < 3; i++) {
        tft.setCursor(110 + i * 40, 253);
        uint32_t v = impedance / divi;
        v *= divi;
        v = impedance - v;
        divi /= 10;
        v /= divi;
        tft.print(v);
      } 
      //dotyk
      if (!ts.read_XY(xy)) {
        touchPressed = 0;
      }
      if (ts.read_XY(xy) && !touchPressed){
        xy[1] = 25 + map(TS_MAXX - xy[1], TS_MINX, TS_MAXX, 0, tft.width());
        xy[0] = map(xy[0], TS_MINY, TS_MAXY, 0, tft.height());
        xy[1] = tft.width() - xy[1];
        xy[0] = tft.height() - xy[0];
        int x = xy[1];
        xy[1] = xy[0];
        xy[0] = x;
        touchPressed = 1;
        //tlacitko ok - primo
        if (xy[0] > 0 && xy[0] < 60 && xy[1] > 95 && xy[1] < 135) {
          mode = 0;
          imped = 0;
        }
        //tlacitko ok - odvodit
        if (xy[0] > 0 && xy[0] < 60 && xy[1] > 195 && xy[1] < 235) {
          mode = 0;
          imped = 1;
        }
        //volba impedance
        //+
        if (xy[1] > 218 && xy[1] < 248) {
          //100
          if (xy[0] > 105 && xy[0] < 135) {
            impedance = 100;
          }
          //10
          if (xy[0] > 145 && xy[0] < 175) {
            if (impedance <= 90) impedance += 10;
          }
          //1
          if (xy[0] > 185 && xy[0] < 215) {
            if (impedance <= 99) impedance += 1;
          }
        }
        //-
        if (xy[1] > 278 && xy[1] < 308) {
          //100
          if (xy[0] > 105 && xy[0] < 135) {
            impedance = 1;
          }
          //10
          if (xy[0] > 145 && xy[0] < 175) {
            if (impedance >= 11) impedance -= 10;
            else impedance = 1;
          }
          //1
          if (xy[0] > 185 && xy[0] < 215) {
            if (impedance >= 2) impedance -= 1;
          }
        }
        
      }
    } 
  }
  
  float genLowRef = 30.79; //rms referencni hodnota napeti generatoru pri urovni 3

  //obnov stranku
  tft.fillRect(0, 40, 240, 280, ILI9341_BLACK);
  
  //zmer vychozi hodnotu zesileni
  tft.setTextColor(ILI9341_YELLOW);
  tft.setCursor(20, 50);
  tft.print("Nastav hlas.");
  tft.setCursor(20, 70);
  tft.print("na maximum");
  //tlacitko ok
  tft.fillRoundRect(0, 95, 60, 40, 5, ILI9341_BLUE);
  tft.setTextColor(ILI9341_WHITE);
  tft.setCursor(6, 100);
  tft.print("OK");
  
  setGen(1, 1000.0, 3);
  
  regWrite(in_md, 1);
  delay(500);

  bool gainF = 1;
  while(gainF){
    //dotyk, tlacitko ok
    if (ts.read_XY(xy)){
      xy[1] = 25 + map(TS_MAXX - xy[1], TS_MINX, TS_MAXX, 0, tft.width());
      xy[0] = map(xy[0], TS_MINY, TS_MAXY, 0, tft.height());
      xy[1] = tft.width() - xy[1];
      xy[0] = tft.height() - xy[0];
      int x = xy[1];
      xy[1] = xy[0];
      xy[0] = x;
      if (xy[0] > 0 && xy[0] < 60 && xy[1] > 95 && xy[1] < 135) {
        gainF = 0;
      }
    }
  }
  //obnov stranku
  tft.fillRect(0, 40, 240, 280, ILI9341_BLACK);
  
  ACrmsvolts = getOutVolt(0);
  
  float gainRef = float(ACrmsvolts) / genLowRef;

  //priprava na hledani orezu
  tft.setTextColor(ILI9341_YELLOW);
  tft.setCursor(20, 50);
  tft.print("Nastav hlas.");
  tft.setCursor(20, 70);
  tft.print("na minimum");
  //tlacitko ok
  tft.fillRoundRect(0, 95, 60, 40, 5, ILI9341_BLUE);
  tft.setTextColor(ILI9341_WHITE);
  tft.setCursor(6, 100);
  tft.print("OK");

  bool waitF = 1;
  while(waitF){
    //dotyk, tlacitko ok
    if (ts.read_XY(xy)){
      xy[1] = 25 + map(TS_MAXX - xy[1], TS_MINX, TS_MAXX, 0, tft.width());
      xy[0] = map(xy[0], TS_MINY, TS_MAXY, 0, tft.height());
      xy[1] = tft.width() - xy[1];
      xy[0] = tft.height() - xy[0];
      int x = xy[1];
      xy[1] = xy[0];
      xy[0] = x;
      if (xy[0] > 0 && xy[0] < 60 && xy[1] > 95 && xy[1] < 135) {
        waitF = 0;
      }
    }
  }
  //obnov stranku
  tft.fillRect(0, 40, 240, 280, ILI9341_BLACK);
  
  //hledani urovne orezu a ideálni amplitudy pro provedeni mereni
  tft.setTextColor(ILI9341_YELLOW);
  tft.setCursor(5, 50);
  tft.print("Hledani orezu");

  delay(500);
  
  byte amplitude = 100;
  regWrite(in_md, 1);
  setGen(1, 1000.0, amplitude);
  delay(500);
  
  //ramecek stinitka
  tft.drawRect(0, RetStartY, RetWidth, RetHeight, RetColor);
  //tlacitko ok
  tft.fillRoundRect(0, 255, 60, 40, 5, ILI9341_BLUE);
  tft.setTextColor(ILI9341_WHITE);
  tft.setCursor(6, 260);
  tft.print("OK");
  
  tft.setTextColor(ILI9341_YELLOW);
  tft.setCursor(5, 232);
  tft.print("Hledej orez");
  
  osciTimeDiv = 3;
  osciVoltDiv = 0;

  //cyklus zobrazeni vystupniho prubehu ze zesilovace
  bool clip = 1;
  while(clip){
    osciSampleVal();
    
    //automaticke prepinani citlivosti osciloskopu
    byte maxi = 0;
    for (int i = 0; i < 119; i++) {
      int val = abs(int(osciDisplay[i]) - 89);
      if(val > maxi) maxi = val;
    }
    if((maxi > 80) && osciVoltDiv != 6) osciVoltDiv++;
    if((maxi < 20) && osciVoltDiv != 0) osciVoltDiv--;

    if(osciVoltDiv > 0){
      float scale = 80.0 / float(maxi);
      for (int i = 0; i < 119; i++) {
        float val = osciDisplay[i] - 89;
        val *= scale;
        val += 0.5 + 89.0;
        osciDisplay[i] = val;
      }
    }
    
    drawGraticule();
    drawOsci();

    //dotyk, tlacitko ok
    if (ts.read_XY(xy)){
      xy[1] = 25 + map(TS_MAXX - xy[1], TS_MINX, TS_MAXX, 0, tft.width());
      xy[0] = map(xy[0], TS_MINY, TS_MAXY, 0, tft.height());
      xy[1] = tft.width() - xy[1];
      xy[0] = tft.height() - xy[0];
      int x = xy[1];
      xy[1] = xy[0];
      xy[0] = x;
      if (xy[0] > 0 && xy[0] < 60 && xy[1] > 255 && xy[1] < 295) {
        clip = 0;
      }
    }
  }
  
  //obnoveni stranky
  tft.fillRect(0, 40, 240, 280, ILI9341_BLACK);
  
  ACrmsvolts = getOutVolt(0);
  
  float baseLvl = ACrmsvolts / gainRef;

  //zmer zesileni a vystupni vykon
  tft.setTextColor(ILI9341_YELLOW);
  tft.setCursor(10, 50);
  tft.print("Zisk+Watt");
  file.println("Zisk");
  
  //zdroj -> ext
  regWrite(in_md, 1);
  delay(100);
  
  ampRange = 0;
  multiSampleVal(0);
  
  gain = gainRef;

  fileprintc(gain, 2);
  file.println();
  fileprintc(20 * 0.4343 * log(gain), 1);
  file.println("dB");

  float voltOut = baseLvl * gainRef;

  if(imped == 1){
    float z = impedance;
    Watt = sq(voltOut) / (z * 1000);
  }
  
  //Watt
  file.println("Vykon");
  if (Watt > 999.00 || Watt < -999.00) {
    fileprintc(Watt / 1000.0, 2);
    file.println("W");
    }
    else {
    fileprintc(Watt, 1);
    file.println("mW");
  }
  //vstupni napeti
  file.println("Napeti na vstupu");
  if(baseLvl > 999.00) {
    fileprintc(baseLvl / 1000.0, 2);
    file.println("V");
  }
  else {
    fileprintc(baseLvl, 1);
    file.println("mV");
  }

  //vystupni napeti
  file.println("Napeti na vystupu");
  if((baseLvl * gainRef) > 999.00) {
    fileprintc(voltOut / 1000.0, 2);
    file.println("V");
  }
  else {
    fileprintc(voltOut, 1);
    file.println("mV");
  }
   
      
  delay(500);
  
  //mereni zkresleni
  tft.fillRect(0, 50, 240, 30, ILI9341_BLACK);
  tft.setCursor(20, 50);
  tft.print("Zkresleni");

  setGen(3, 1000.0, amplitude);
  delay(100);
  
  //zdroj -> ext
  regWrite(in_md, 1);
  regWrite(in_en, 1);
  delay(100);
  autoRangevolt(0);
  distort = 0.0;
  distortN = 0.0;
  for(int i = 0; i < 10; i++){
    takeSamples(0, 0, 2);
    distort += THDanalyze(20, 0);
    distortN += THDanalyze(20, 1);
  }
  regWrite(in_en, 0);
  
  distort /= 10.0;
  distortN /= 10.0;
  //zkresleni
  file.println("Zkresleni");
  fileprintc(distort * 100.0, 2);
  file.println("%");

  distort = 20 * 0.4343 * log(distort);
    
  fileprintc(distort, 1);
  file.println("dB");
      
  //zkresleni a sum
  file.println("Zkresleni a sum");
  fileprintc(distortN * 100.0, 2);
  file.println("%");

  distortN = 20 * 0.4343 * log(distortN);
  
  fileprintc(distortN, 1);
  file.println("dB");
  
  //mereni frekvencni amplitudove charakteristiky
  tft.fillRect(0, 50, 240, 30, ILI9341_BLACK);
  tft.setCursor(10, 50);
  tft.print("Frek char");
  file.println();
  file.println("Frek. char.");

  //zahlavi tabulky
  file.print("f / kHz");
  file.print(separ);
  file.print("Au / -");
  file.print(separ);
  file.println("au / dB");
  uint32_t decade = 10;
  while(decade != 100000){
    byte start = 1;
    if(decade == 10)start = 3;

    for(byte i = start; i < 10; i++){
      float fr = decade * logScale[i];
      //nastaveni generatoru
      setGen(1, fr, amplitude);
      bool t = 0;
      //zapis frekvence
      if(decade < 1000){
        if(decade < 100) delay(1000);
        t = 1;
        fileprintc(fr / 1000.0, 3);
      }
      else fileprintc(fr / 1000.0, 1);

      //zmereni vystupni amplitudy
      float outVolt = getOutVolt(t);

      //spocitani zesileni
      gain = outVolt / baseLvl;

      //zobrazeni frekvence na TFT
      tft.fillRect(0, 75, 240, 30, ILI9341_BLACK);
      tft.setCursor(10, 75);
      if (fr > 999.00) {
        tft.print(fr / 1000.0, 1);
        tft.print("kHz");
      }
      else {
        tft.print(fr, 1);
        tft.print("Hz");
      }

      //zapis zesileni a zisku
      file.print(separ);
      fileprintc(gain, 2);
      file.print(separ);
      gain = 20 * 0.4343 * log(gain);
      fileprintc(gain, 1);
      file.println();

      //zobrazeni zisku na TFT
      tft.fillRect(0, 100, 240, 30, ILI9341_BLACK);
      tft.setCursor(10, 100);
      tft.print(gain, 1);
      tft.print("dB");
    }
    decade *= 10;
  }

  //obnoveni stranky
  tft.fillRect(0, 40, 240, 280, ILI9341_BLACK);
  
  //mereni sumoveho napeti
  tft.fillRect(0, 50, 240, 30, ILI9341_BLACK);
  tft.setCursor(20, 50);
  tft.print("Sum napeti");
  file.println();
  file.println("Sumove napeti");

  float Noise = getNoise();
  
  fileprintc(Noise, 1);
  file.println("mV");

  file.println();

  tft.fillRect(0, 50, 240, 30, ILI9341_BLACK);
  tft.setCursor(20, 50);
  tft.print("Hotovo");
  
  delay(500);
  tft.fillRect(0, 140, 240, 30, ILI9341_BLACK);

  //nahraj data do karty SD
  if (!file.sync() || file.getWriteError()) {
    Serial.println("write error");
  }
}

//poloautomaticky test - frekvencni charakteristika s volbou amplitudy generatoru
void FreqResp(){
  byte amplitude = 100;
  
  tft.setTextColor(ILI9341_YELLOW);
  tft.setTextSize(3);

  //obnov stranku
  tft.fillRect(0, 40, 240, 280, ILI9341_BLACK);
  
  //spusteni testu
  tft.fillRect(0, 140, 240, 30, ILI9341_BLACK);
  tft.setCursor(10, 140);
  tft.print("Testuji");

  delay(500);

  //obnov stranku
  tft.fillRect(0, 40, 240, 280, ILI9341_BLACK);

  //volba amplitudy mereni
  tft.setCursor(10, 50);
  tft.print("Mereni");
  tft.setCursor(10, 70);
  tft.print("Frek char.");
  //tlacitko ok - primo
  tft.fillRoundRect(0, 95, 60, 40, 5, ILI9341_BLUE);
  tft.setTextColor(ILI9341_WHITE);
  tft.setCursor(6, 100);
  tft.print("OK");

  tft.setTextColor(ILI9341_YELLOW);
  tft.setCursor(10, 145);
  tft.print("zadejte");
  tft.setCursor(10, 170);
  tft.print("amplitudu");

  //zadani amplitudy
  tft.setTextColor(ILI9341_WHITE);
  tft.setTextSize(3);
  tft.setCursor(10, 240);
  tft.print("Ampli");
  tft.setCursor(10, 265);
  tft.print("  V");

  //vykresleni tlacitek + a -
  for (int d = 0; d < 2; d++) {
    for (int i = 0; i < 3; i++) {
      tft.fillRoundRect(105 + i * 40, 218 + d * 60, 30, 30, 5, ILI9341_BLUE);
      tft.setCursor(108 + i * 40 + 5, 223 + d * 60);
      if (d == 1) tft.print('-');
      else tft.print('+');
    }
  }

  bool mode = 1;
  while(mode){
    //zobrazeni zadani hodnot impedance
    if (millis() > (multiLimit + 50)) {
      multiLimit = millis();
      tft.setTextColor(ILI9341_WHITE);
      tft.fillRect(110, 253, 100, 25, ILI9341_BLACK);
      bool rg = 0;
      tft.setCursor(130, 253);
      if(amplitude > 100){
        rg = 1;
        tft.setCursor(170, 253);
      }
      tft.print('.');
      int divi = 1000;
      for (int i = 0; i < 3; i++) {
        tft.setCursor(110 + i * 40, 253);
        uint32_t v = (amplitude - 91 * rg) / divi;
        v *= divi;
        v = (amplitude - 91 * rg) - v;
        divi /= 10;
        v /= divi;
        tft.print(v);
      } 
      //dotyk
      if (!ts.read_XY(xy)) {
        touchPressed = 0;
      }
      if (ts.read_XY(xy) && !touchPressed){
        xy[1] = 25 + map(TS_MAXX - xy[1], TS_MINX, TS_MAXX, 0, tft.width());
        xy[0] = map(xy[0], TS_MINY, TS_MAXY, 0, tft.height());
        xy[1] = tft.width() - xy[1];
        xy[0] = tft.height() - xy[0];
        int x = xy[1];
        xy[1] = xy[0];
        xy[0] = x;
        touchPressed = 1;
        //tlacitko ok
        if (xy[0] > 0 && xy[0] < 60 && xy[1] > 95 && xy[1] < 135) {
          mode = 0;
        }
        //volba impedance
        //+
        if (xy[1] > 218 && xy[1] < 248) {
          //100
          if (xy[0] > 105 && xy[0] < 135) {
            if(rg == 1) amplitude = 191;
            else amplitude = 101;
          }
          //10
          if (xy[0] > 145 && xy[0] < 175) {
            if ((amplitude - rg * 91) <= 90) amplitude += 10;
            else if(rg == 1) amplitude = 191;
            else if(rg == 0) amplitude = 101;
          }
          //1
          if (xy[0] > 185 && xy[0] < 215) {
            if ((amplitude - rg * 91) <= 99) amplitude += 1;
            else if(rg == 0) amplitude = 101;
          }
        }
        //-
        if (xy[1] > 278 && xy[1] < 308) {
          //100
          if (xy[0] > 105 && xy[0] < 135) {
            if(rg == 1) amplitude = 100;
            else amplitude = 0;
          }
          //10
          if (xy[0] > 145 && xy[0] < 175) {
            if(rg == 1 && (amplitude - 91) >= 19) amplitude -= 10;
            else if(rg == 1) amplitude = 100;
            if (rg == 0 && amplitude >= 10) amplitude -= 10;
            else if(rg == 0) amplitude = 0;
          }
          //1
          if (xy[0] > 185 && xy[0] < 215) {
            if ((amplitude - rg * 91) >= 1) amplitude -= 1;
          }
        } 
      }
    } 
  }
  
  
  //obnov stranku
  tft.fillRect(0, 40, 240, 280, ILI9341_BLACK);
  
  setGen(1, 1000.0, amplitude);
  regWrite(in_md, 0);
  delay(100);
  float baseLvl = getOutVolt(0);

  regWrite(in_md, 1);
  delay(100);
  
  //mereni frekvencni amplitudove charakteristiky
  tft.fillRect(0, 50, 240, 30, ILI9341_BLACK);
  tft.setTextColor(ILI9341_YELLOW);
  tft.setCursor(10, 50);
  tft.print("Frek char");
  file.println();
  file.println("Frek. char.");

  //zahlavi tabulky
  file.print("f / kHz");
  file.print(separ);
  file.print("Au / -");
  file.print(separ);
  file.println("au / dB");
  uint32_t decade = 10;
  while(decade != 100000){
    byte start = 1;
    if(decade == 10)start = 3;

    for(byte i = start; i < 10; i++){
      float fr = decade * logScale[i];
      //nastaveni generatoru
      setGen(1, fr, amplitude);
      bool t = 0;
      //zapis frekvence
      if(decade < 1000){
        if(decade < 100) delay(1000);
        t = 1;
        fileprintc(fr / 1000.0, 3);
      }
      else fileprintc(fr / 1000.0, 1);

      //zmereni vystupni amplitudy
      float outVolt = getOutVolt(t);

      //spocitani zesileni
      gain = outVolt / baseLvl;

      //zobrazeni frekvence na TFT
      tft.fillRect(0, 75, 240, 30, ILI9341_BLACK);
      tft.setCursor(10, 75);
      if (fr > 999.00) {
        tft.print(fr / 1000.0, 1);
        tft.print("kHz");
      }
      else {
        tft.print(fr, 1);
        tft.print("Hz");
      }

      //zapis zesileni a zisku
      file.print(separ);
      fileprintc(gain, 2);
      file.print(separ);
      gain = 20 * 0.4343 * log(gain);
      fileprintc(gain, 1);
      file.println();

      //zobrazeni zisku na TFT
      tft.fillRect(0, 100, 240, 30, ILI9341_BLACK);
      tft.setCursor(10, 100);
      tft.print(gain, 1);
      tft.print("dB");
    }
    decade *= 10;
  }

  //obnoveni stranky
  tft.fillRect(0, 40, 240, 280, ILI9341_BLACK);

  file.println();

  tft.fillRect(0, 50, 240, 30, ILI9341_BLACK);
  tft.setCursor(20, 50);
  tft.print("Hotovo");
  
  delay(500);
  tft.fillRect(0, 140, 240, 30, ILI9341_BLACK);

  //nahraj data do karty SD
  if (!file.sync() || file.getWriteError()) {
    Serial.println("write error");
  }
}

//funkce ovladani pinu registru vystupniho rozsireni
void regWrite(byte pin, bool val) {
  bitWrite(regOut, pin, val);
  for (byte i = 0; i < 8; i++)  {
    digitalWrite(regData, !!(regOut & (1 << (7 - i))));
    digitalWrite(regClk, HIGH);
    digitalWrite(regClk, LOW);
  }
  digitalWrite(regData, LOW);
  digitalWrite(regLoad, LOW);
  digitalWrite(regLoad, HIGH);
}

//funkce nastaveni vstupniho zesileni
void vInSetLevel(byte setlevel) {
  if (aLvTable[setlevel] > 99) {
    inputRange = 0;
    aLv.setPot(aLvTable[setlevel] - 100, 0);

  }
  else {
    inputRange = 1;
    aLv.setPot(aLvTable[setlevel], 0);
  }
  if (inputRange != curinputRange) {
    regWrite(in_vR, inputRange);
    curinputRange = inputRange;
    delay(1);
  }
  aLevel = setlevel;
}

//funkce vlastni kalibrace
void selfCalibrate(){
  //nastaveni generatoru pro generaci 7.071 Vrms 1kHz sinusoveho prubehu
  curgenMode++;
  curgenFreq++;
  curgenLevel++;
  regWrite(gen_en, 0);
  regWrite(gen_md, 0);
  regWrite(gen_rg, 1);
  gen.setMode(MD_AD9833::MODE_SINE);
  gen.setFrequency(MD_AD9833::CHAN_0, 1000);
  gLv.setPot(99, 0);
  
  //zdroj -> gen
  regWrite(in_md, 0);
  delay(50);
  //nulovani a aktivace vstupu
  VoltNull();
  regWrite(in_en, 1);
  //cekani stability
  delay(1000);
  //mereni vice vzorku
  autoRangevolt(0);
  uint64_t voltAC = 0;
  for(uint16_t c = 0; c < 10; c++){
    takeSamples(0, 0, 1);
    for (uint16_t i = 0; i < maxSamples; i++) {
      voltAC += sq(dataPoints[i] - nullpoint);
    }
  }
  regWrite(in_en, 0);
  regWrite(in_md, osciSource);

  //vypocteni prumerne rms hodnoty
  ACrmsvolts = (double)voltAC;
  ACrmsvolts /= double(10 * maxSamples); 
  ACrmsvolts = sqrt(ACrmsvolts);
  ACrmsvolts /= (aVoltDivTable[aLevel] * calibrate);
  //umerna zmena kalibracni konstanty
  calibrate *= ACrmsvolts / 7071.0;
}

//vzorkovani napeti, proudu a vykonu do zateze
void multiSampleVal(bool t) {
  regWrite(in_en, 1);
  delay(1);
  autoRangevolt(t);
  takeSamples(0, t, 0);
  regWrite(in_en, 0);
  
  byte border = 0;
  if(t == 1) border = 10;
  
  int64_t volt = 0;
  for (uint16_t i = border; i < maxSamples - border; i++) {
    volt += dataPoints[i] - nullpoint;
  }
  DCvolts = (double)volt;
  DCvolts /= maxSamples;
  DCvolts /= (aVoltDivTable[aLevel] * calibrate);
  volt = 0;
  for (uint16_t i = border; i < maxSamples - border; i++) {
    volt += sq(dataPoints[i] - nullpoint);
  }
  ACrmsvolts = (double)volt;
  ACrmsvolts /= maxSamples; 
  ACrmsvolts = sqrt(ACrmsvolts);
  ACrmsvolts /= (aVoltDivTable[aLevel] * calibrate);

  //citlivost ampermetru
  if (ampRange != curampRange) {
    regWrite(in_aR, ampRange);
    curampRange = ampRange;
    delay(20);
  }
  takeSamples(1, t, 0);
  int64_t amp = 0;
  for (uint16_t i = border; i < maxSamples - border; i++) {
    amp += dataPoints[i] - nullAmp;
  }
  DCamps = (double)amp;
  DCamps /= maxSamples;
  DCamps /= ampDiv[ampRange];
  amp = 0;
  for (uint16_t i = border; i < maxSamples - border; i++) {
    amp += sq(dataPoints[i] - nullAmp);
  }
  ACrmsamps = (double)amp;
  ACrmsamps /= maxSamples; 
  ACrmsamps = sqrt(ACrmsamps);
  ACrmsamps /= ampDiv[ampRange];

  Watt = ACrmsvolts * ACrmsamps / 1000.0;
  
  ACrmsvolts -= abs(DCvolts);
}

//nulovani napetove odchylky
void VoltNull() {
  takeSamples(0, 0, 0);
  uint32_t nul = 0;
  for (uint16_t i = 0; i < maxSamples; i++) {
    nul += dataPoints[i];
  }
  double nula = (double)nul;
  nula /= maxSamples;
  nullpoint = (uint16_t)nula;
  if (0.5 < (nula - (double)nullpoint)) nullpoint++;
}

//nulovani proudove odchylky
void AmpNull(){
  takeSamples(1, 0, 0);
  uint32_t nul = 0;
  for (uint16_t i = 0; i < maxSamples; i++) {
    nul += dataPoints[i];
  }
  double nula = (double)nul;
  nula /= maxSamples;
  nullAmp = (uint16_t)nula;
  if (0.5 < (nula - (double)nullpoint)) nullAmp++;
}

//vzorkovani a skalovani hodnot osciloskopu
void osciSampleVal() {
  regWrite(in_en, 1);
  vInSetLevel(osciVoltLevelTable[osciVoltDiv]);
  delayMicroseconds(500);
  if(osciTimeDiv > 4) tB = 1;
  else tB = 0;
  
  takeSamples(0, tB, 1);
  regWrite(in_en, 0);
  
  double timemult = (double)osciTimeDivTable[osciTimeDiv] / sRateTable[tB];
  double voltmult = 17.8 / ((double)osciVoltDivTable[osciVoltDiv] * calibrate * aVoltDivTable[aLevel]);
  
  for (uint16_t i = 0; i < 119; i++) {
    double pos = i * timemult;
    uint16_t index = (int)pos;
    double interpolate = pos - (double)index;
    double val = (1.0 - interpolate) * dataPoints[index] + interpolate * dataPoints[index + 1];
    val -= nullpoint;
    val *= voltmult;
    if(val - (double)((int)val) > 0.5) val++;
    if (val > 89.00) val = 89.00;
    if (val < -89.00) val = -89.00;
    val += 89.00;
    osciDisplay[i] = val;
  }
}

//vykresleni stinitka osciloskopu
void drawGraticule() {
  tft.fillRect(1, RetStartY + 1, RetWidth - 2, RetHeight - 2, ILI9341_BLACK);
  for (uint16_t TicksX = 1; TicksX < 10; TicksX++)
  {
    for (uint16_t TicksY = 1; TicksY < 10; TicksY++)
    {
      tft.drawPixel(  TicksX * (RetWidth / 10), RetStartY + TicksY * (RetHeight / 10), RetColor);
    }
  }
  for (uint16_t TicksX = 0; TicksX < RetWidth; TicksX += (RetWidth / 50))
  {
    if (TicksX % (RetWidth / 10) > 0 )
    {
      tft.drawFastVLine(TicksX, RetStartY + (RetHeight / 2) - 1, 3, RetColor);
    }
    else
    {
      tft.drawFastVLine(TicksX, RetStartY + (RetHeight / 2) - 3, 7, RetColor);
    }

  }

  for (uint16_t TicksY = RetStartY; TicksY < (RetHeight + RetStartY); TicksY += (RetHeight / 50))
  {
    if ((TicksY - RetStartY) % (RetHeight / 10) > 0 )
    {
      tft.drawFastHLine((RetWidth / 2) - 1, TicksY, 3, RetColor);
    }
    else
    {
      tft.drawFastHLine((RetWidth / 2) - 3, TicksY, 7, RetColor);
    }
  }
}

//vykres prubehu napeti na osciloskop
void drawOsci() {
  for (int i = 0; i < ((RetWidth - 2) / 2) - 1; i++) {
    tft.drawLine(2 * i + 1, (RetStartY + RetHeight - 2) - osciDisplay[i], 2 * i + 3, (RetStartY + RetHeight - 2) - osciDisplay[i + 1], beamColor);
  }
}

//funkce automatickeho prepnuti vstupniho rozsahu
void autoRangevolt(bool t) {
  bool autorange = 1;
  bool maximize = 1;
  vInSetLevel(29);
  byte border = 0;
  if(t == 1) border = 10;
  while (autorange || maximize) {
    takeSamples(0, t, 0);
    uint16_t maximum = 0;
    maximize = 1;
    for (uint16_t i = border; i < maxSamples - border; i++) {
      int16_t data = dataPoints[i] - nullpoint;
      if (data < 0) data = -data;
      if (maximum < data) maximum = data;
    }
    if (maximum > 2040 && aLevel < (aSteps - 1)) {
      aLevel++;
      vInSetLevel(aLevel);
    }
    else maximize = 0;

    if ((maximum < 1700) && aLevel > 0) {
      aLevel--;
      vInSetLevel(aLevel);
    }
    else autorange = 0;
  }
}

//zmereni napeti na vystupu
float getOutVolt(bool t){
  curosciSource = !curosciSource;
  //nulovani a spusteni vstupu
  VoltNull();
  regWrite(in_en, 1);
  //source -> ext
  regWrite(in_md, 1);
  delay(200);
  //sber vice vzorku
  autoRangevolt(t);
  uint64_t voltAC = 0;
  for(uint16_t c = 0; c < 10; c++){
    takeSamples(0, t, 1);
    for (uint16_t i = 0; i < maxSamples; i++) {
      voltAC += sq(dataPoints[i] - nullpoint);
    }
  }
  //vypocet rms hodnoty
  double rms = (double)voltAC;
  rms /= double(10 * maxSamples); 
  rms = sqrt(rms);
  rms /= (aVoltDivTable[aLevel] * calibrate);
  
  regWrite(in_en, 0);
  return float(rms);
}

//funkce pro mereni vystupniho sumoveho napeti
float getNoise(){
  setGen(0, 0, 0);
  //zdroj -> ext
  regWrite(in_md, 1);
  regWrite(in_en, 1);
  delay(100);
  
  float noiseRMS = 0;
  for(byte d = 0; d < 10; d++) {
  ACrmsvolts = getOutVolt(1);
  noiseRMS += ACrmsvolts;
  }
  
  noiseRMS /= 10;
  regWrite(in_en, 0);
  return noiseRMS;
}

//funkce pro zapis dessetinych cisel na kartu SD s carkou jako oddelovacem dessetin
void fileprintc(float val, uint32_t dec){
  char buffer[] = "99999.999";
  
  dtostrf(val, 0, dec, buffer);
  char* p  = strchr(buffer, '.');
  *p = ',';
  file.print(buffer);
}

//vypocet harmonickeho zkresleni
float THDanalyze(byte HarmonicsCount, bool type) {
  uint16_t outputSampleCount = 2 * HarmonicsCount + 2;
  float sampledData[42];
  
  //vycentruj
  int16_t minimum = 0;
  int16_t maximum = 0;
  for (uint16_t i = 0; i < maxSamples; i++) {
    if (minimum > dataPoints[i]) minimum = dataPoints[i];
    if (maximum < dataPoints[i]) maximum = dataPoints[i];
  }
  float cen = (maximum + minimum) / 2.0 + 0.5;
  minimum = 0;
  maximum = 0;
  for (uint16_t i = 0; i < maxSamples; i++) {
    dataPoints[i] -= cen;
    if (minimum > dataPoints[i]) minimum = dataPoints[i];
    if (maximum < dataPoints[i]) maximum = dataPoints[i];
  }

  //najdi spousteci body pro vyber periody prubehu
  uint16_t tP1 = 1;
  uint16_t tP2 = 1;
  uint16_t tP3 = 1;

  uint16_t tSampleWidth = (maxSamples / 1250) + 1;
  
  for (uint16_t i = tSampleWidth; i < maxSamples - tSampleWidth; i++) {
    float a = 0;
    float b = 0;
    for (uint16_t c = i - tSampleWidth; c < i; c++) {
      a += dataPoints[c];
    }
    for (uint16_t c = i; c < i + tSampleWidth; c++) {
      b += dataPoints[c];
    }
    a /= tSampleWidth;
    b /= tSampleWidth;
    if ((a < 0) && (b > 0) && (tP1 == 1)) tP1 = i;
    if ((a > 0) && (b < 0) && (tP1 != 1) && (i > (tP1 + 10)) && (tP2 == 1)) tP2 = i;
    if ((a < 0) && (b > 0) && (tP1 != 1) && (i > (tP2 + 10)) && (tP2 != 1) && (tP3 == 1))tP3 = i;
  }

  //v pripade nenalezeni prubehu ukoncit funkci
  if((tP1 < 2) || (tP2 < 5) || (tP3 < 10)) return 1.0;

  //delka vybraneho prubehu
  uint16_t delka = tP3 - tP1;

  //supersampling
  float sampleStep = delka / float(outputSampleCount);
  uint16_t sampleSize = sampleStep / 2;
  for (uint16_t i = 0; i < outputSampleCount; i++) {
    //zvol dve okoli a pocitej dva prumery
    float sample1 = 0;
    float sample2 = 0;
    float step = i * sampleStep;
    for (uint16_t c = tP1 + uint16_t(step) - sampleSize; c <= tP1 + uint16_t(step) + sampleSize; c++) {
      sample1 += dataPoints[c];
      sample2 += dataPoints[c + 1];
    }
    sample1 /= 2 * sampleSize + 1;
    sample2 /= 2 * sampleSize + 1;
    //skalovana interpolace
    float dif = step - float(uint16_t(step));
    sampledData[i] = sample1 * (1.0 - dif) + sample2 * dif;
  }
  
  if(type == 1) HarmonicsCount = 1;
  
  //harmonicka analyza
  float prvniHa = 0;
  float prvniHb = 0;
  float SUMostatniH = 0;
  for (uint16_t h = 0; h < HarmonicsCount; h++) {
    float a = 0;
    float b = 0;
    for (uint16_t i = 0; i < outputSampleCount; i++) {
      float rad = ((h + 1) * 6.28318531) * (float(i) / outputSampleCount);
      a += sampledData[i] * cos(rad);
      b += sampledData[i] * sin(rad);
    }
    a /= (outputSampleCount / 2);
    b /= (outputSampleCount / 2);
    if (h == 0) {
      prvniHa = a;
      prvniHb = b;
    }
    else SUMostatniH += sq(a) + sq(b);
  }
  SUMostatniH = sqrt(SUMostatniH);
  
  float prvniH = sqrt(sq(prvniHa) + sq(prvniHb));
  if(type == 0){
    //analyza harmonickeho zkresleni
    float THD = (SUMostatniH / prvniH);
    if(THD > 1.0) THD = 1.0;
    return THD;
  }
  if(type == 1) {
    //analyza harmonickeho zkresleni se sumem
    float noiseRMS = 0;
    for (uint16_t i = 0; i < outputSampleCount; i++) {
      float rad = 6.28318531 * (float(i) / outputSampleCount);
      float noiseData = prvniHa * cos(rad) + prvniHb * sin(rad);
      noiseData -= sampledData[i];
      noiseRMS += sq(noiseData);
    }
    noiseRMS = sqrt(noiseRMS / outputSampleCount);
    float THD_N = noiseRMS / (prvniH / 1.41421356);
    if(THD_N > 1.0) THD_N = 1.0;
    return THD_N;
  }
  return 0;
}

//funkce pro mereni frekvence, stridy a periody vstupního signálu
void getFreq(){
  double t1 = 0;
  double t2 = 0;
  uint32_t timeout = 0;
  //spoust
  while (digitalRead(vTriggerPin) == 1) {
    if (timeout > 65530) break;
    timeout++;
  }
  timeout = 0;
  while (digitalRead(vTriggerPin) == 0) {
    if (timeout > 65530) break;
    timeout++;
  }
  //kladna cast prubehu
  t1 = micros();
  t2 = t1;
  while (digitalRead(vTriggerPin) == 1) {
    if (timeout > 65530) break;
    timeout++;
  }
  t1 = (micros() - t1);
  //zaporna cast prubehu
  timeout = 0;
  while (digitalRead(vTriggerPin) == 0) {
    if (timeout > 65530) break;
    timeout++;
  }
  t2 = (micros() - t2);
  
  //vypocet
  if(t2 > 51000) {
    period = 0.0;
    duty = 0.5;
    freq = 0;
  }
  else {
    period = float(t2);
    duty = float(t1 / t2);
    freq = 1000000.0 / t2;
  }
}

//nezavisla funkce pro rychle nastaveni generatoru
void setGen(byte mode, float fr, byte level){
  if (mode != curgenMode) {
    switch (mode) {
      case (0):
        gen.setMode(MD_AD9833::MODE_OFF);
        regWrite(gen_en, 0);
        regWrite(gen_md, 0);
        gen.setFrequency(MD_AD9833::CHAN_0, genFreq);
        break;
      case (1):
        gen.setMode(MD_AD9833::MODE_SINE);
        regWrite(gen_en, 1);
        regWrite(gen_md, 0);
        gen.setFrequency(MD_AD9833::CHAN_0, genFreq);
        break;
      case (2):
        gen.setMode(MD_AD9833::MODE_TRIANGLE);
        regWrite(gen_en, 1);
        regWrite(gen_md, 0);
        gen.setFrequency(MD_AD9833::CHAN_0, genFreq);
        break;
      case (3):
        gen.setMode(MD_AD9833::MODE_SINE);
        regWrite(gen_en, 1);
        regWrite(gen_md, 1);
        gen.setFrequency(MD_AD9833::CHAN_0, k_ref);
        curgenFreq = k_ref;
        break;
    }
    curgenMode = mode;
  }
  
  if (fr != curgenFreq && mode != 3) {
    gen.setFrequency(MD_AD9833::CHAN_0, fr);
    curgenFreq = fr;
  }
  if (level != curgenLevel) {
    if(level > 191) level = 191;
    curgenLevel = level;
    if (level == 0) {
      gLv.setPot(0, 0);
      regWrite(gen_en, 0);
    }
    else {
      if (mode != 0) regWrite(gen_en, 1);
      if (level > 100){
        regWrite(gen_rg, 1);
        if(genRange != 1) delay(10);
        genRange = 1;
      }
      else {
        regWrite(gen_rg, 0);
        if(genRange != 0) delay(10);
        genRange = 0;
      }
      gLv.setPot(level - 1 - (genRange * 90), 0);
    }
  }
}

//funkce pro sber vzorku
//tato funkce pouziva dvoji stridave vzorkovani s primym pristupem do pameti pro maximalizaci rychlosti ADC
void takeSamples(bool source, bool delayStep, byte triggerMode) {
  //             0 napeti     casova zakladna      0 bez spouste, 1 nabezna hrana, 2 klesajici hrana
  //             1 proud 
  int pinMapADCin = 0;
  int triggerPin = 0;
  
  //vzorkuj proud
  if(source == 1) {
    pinMapADCin = PIN_MAP[analogInPinI].adc_channel;
    triggerPin = aTriggerPin;
  }
  //vzorkuj napeti
  else {
    pinMapADCin = PIN_MAP[analogInPin].adc_channel;
    triggerPin = vTriggerPin;
  }
  //casova zakladna
  //pomala
  if(delayStep == 1) {
      adc_set_sample_rate(ADC1, ADC_SMPR_239_5); //10.49us na vzorek
      adc_set_sample_rate(ADC2, ADC_SMPR_239_5);
  }
  //rychla
  else {
      adc_set_sample_rate(ADC1, ADC_SMPR_1_5); //0,58uS na vzorek
      adc_set_sample_rate(ADC2, ADC_SMPR_1_5);
  }

  adc_set_reg_seqlen(ADC1, 1);
  ADC1->regs->SQR3 = pinMapADCin;
  ADC1->regs->CR2 |= ADC_CR2_CONT;
  ADC1->regs->CR1 |= ADC_CR1_FASTINT;
  ADC1->regs->CR2 |= ADC_CR2_SWSTART;

  ADC2->regs->CR2 |= ADC_CR2_CONT;
  ADC2->regs->SQR3 = pinMapADCin;

  dma_init(DMA1);
  dma_attach_interrupt(DMA1, DMA_CH1, DMA1_CH1_Event);

  adc_dma_enable(ADC1);
  dma_setup_transfer(DMA1, DMA_CH1, &ADC1->regs->DR, DMA_SIZE_32BITS, dataPoints32, DMA_SIZE_32BITS, (DMA_MINC_MODE | DMA_TRNS_CMPLT));
  dma_set_num_transfers(DMA1, DMA_CH1, maxSamples / 2);
  dma1_ch1_Active = 1;

  //spousteni
  if(triggerMode != 0) {
    uint16_t timeout = 0;
    bool edge = (triggerMode - 1);
    while (digitalRead(triggerPin) == !edge) {
      if (timeout > 65530) break;
      timeout++;
    }
    timeout = 0;
    while (!digitalRead(triggerPin) == !edge) {
      if (timeout > 65530) break;
      timeout++;
    }
  }

  //spusteni vzorkovani
  dma_enable(DMA1, DMA_CH1);
  //cekani na konec vzorkovani
  while (dma1_ch1_Active);
  //konec vzorkovani
  dma_disable(DMA1, DMA_CH1);
}
//funkce potrebne pro provoz rychleho vzorkovani s primym pristupem do pameti
void adc_dma_enable(const adc_dev * dev) {
  bb_peri_set_bit(&dev->regs->CR2, ADC_CR2_DMA_BIT, 1);
}

void adc_dma_disable(const adc_dev * dev) {
  bb_peri_set_bit(&dev->regs->CR2, ADC_CR2_DMA_BIT, 0);
}

static void DMA1_CH1_Event() {
  dma1_ch1_Active = 0;
}
